<?php
/**
 * "CiviCRM Redirect" ACFE Form Action Class.
 *
 * Handles the "CiviCRM Redirect" ACFE Form Action.
 *
 * @package Conditional_Form_Actions_For_ACFE
 */

// Exit if accessed directly.
defined( 'ABSPATH' ) || exit;

/**
 * CiviCRM Profile Sync "CiviCRM Redirect" ACFE Form Action Class.
 *
 * A class that handles the "CiviCRM Redirect" ACFE Form Action.
 *
 * @since 0.1
 */
class CFAFA_Form_Action_Redirect extends CFAFA_Form_Action_Base {

	/**
	 * Plugin object.
	 *
	 * @since 0.1
	 * @access public
	 * @var Conditional_Form_Actions_For_ACFE
	 */
	public $plugin;

	/**
	 * ACF Extended object.
	 *
	 * @since 0.1
	 * @access public
	 * @var CFAFA_ACFE
	 */
	public $acfe;

	/**
	 * Form Action Name.
	 *
	 * @since 0.1
	 * @access public
	 * @var string
	 */
	public $action_name = 'redirect_cfafa';

	/**
	 * Field Key Prefix.
	 *
	 * @since 0.1
	 * @access public
	 * @var string
	 */
	public $field_key = 'field_cfafa_redirect_';

	/**
	 * Field Name Prefix.
	 *
	 * @since 0.1
	 * @access public
	 * @var string
	 */
	public $field_name = 'cfafa_redirect_';

	/**
	 * Constructor.
	 *
	 * @since 0.1
	 *
	 * @param object $parent The parent object reference.
	 */
	public function __construct( $parent ) {

		// Store references to objects.
		$this->plugin = $parent->plugin;
		$this->acfe   = $parent;

		// Label this Form Action.
		$this->action_label = __( 'Conditional Redirect action', 'conditional-form-actions-for-acfe' );

		// Alias Placeholder for this Form Action.
		$this->alias_placeholder = __( 'Conditional Redirect', 'conditional-form-actions-for-acfe' );

		// Init parent.
		parent::__construct();

	}

	/**
	 * Configure this object.
	 *
	 * @since 0.1
	 */
	public function configure() {

		// Conditional Field.
		$this->mapping_field_filters_add( 'redirect_conditional' );

	}

	/**
	 * Performs the action when the Form the Action is attached to is submitted.
	 *
	 * @since 0.1
	 *
	 * @param array   $form The array of Form data.
	 * @param integer $current_post_id The ID of the Post from which the Form has been submitted.
	 * @param string  $action The customised name of the action.
	 */
	public function make( $form, $current_post_id, $action ) {

		// Bail if a filter has overridden the action.
		if ( false === $this->make_skip( $form, $current_post_id, $action ) ) {
			return;
		}

		// Populate Redirect data array.
		$conditional_redirect = $this->form_redirect_data( $form, $current_post_id, $action );

		// Save the results of this Action for later use.
		$this->make_action_save( $action, $conditional_redirect );

		// Check Conditional.
		if ( ! $this->form_conditional_check( $conditional_redirect ) ) {
			return;
		}

		// Strip out any empty Fields.
		$redirect_data = $this->form_data_prepare( $conditional_redirect );

		// Sanity check.
		if ( empty( $redirect_data['redirect_url'] ) ) {
			return;
		}

		/**
		 * Fire the ACF Extended action.
		 *
		 * This is to allow any plugins that perform administrative tasks (e.g. saving
		 * form submissions) to act before the redirect.
		 *
		 * @since 0.1.1
		 *
		 * @param array   $form The array of Form data.
		 * @param integer $current_post_id The ID of the Post from which the Form has been submitted.
		 */
		do_action( 'acfe/form/submit', $form, $current_post_id );

		// Do the redirect.
		wp_safe_redirect( $redirect_data['redirect_url'] );
		exit;

	}

	/**
	 * Defines additional Fields for the "Action" Tab.
	 *
	 * @since 0.1
	 *
	 * @return array $fields The array of Fields for this section.
	 */
	public function tab_action_append() {

		// Init Fields.
		$fields = [];

		// Add URL Field.
		$fields[] = [
			'key'               => $this->field_key . 'redirect_url',
			'label'             => __( 'Redirect URL', 'conditional-form-actions-for-acfe' ),
			'name'              => $this->field_name . 'redirect_url',
			'type'              => 'text',
			'instructions'      => __( 'The URL to redirect to. See "Cheatsheet" tab for all available template tags.', 'conditional-form-actions-for-acfe' ),
			'required'          => 0,
			'conditional_logic' => 0,
			'wrapper'           => [
				'width'                      => '',
				'class'                      => '',
				'id'                         => '',
				'data-instruction-placement' => 'field',
			],
			'acfe_permissions'  => '',
			'default_value'     => '',
			'placeholder'       => '',
			'prepend'           => '',
			'append'            => '',
			'maxlength'         => '',
		];

		// Add Conditional Field.
		$code                       = 'redirect_conditional';
		$label                      = __( 'Conditional On', 'conditional-form-actions-for-acfe' );
		$conditional                = $this->mapping_field_get( $code, $label );
		$conditional['placeholder'] = __( 'Always redirect', 'conditional-form-actions-for-acfe' );
		$conditional['wrapper']['data-instruction-placement'] = 'field';
		$conditional['instructions']                          = __( 'To redirect only when a Form Field is populated, link this to the Form Field. To redirect only when more complex conditions are met, link this to a Hidden Field with value "1" where the conditional logic of that Field shows it when the conditions are met.', 'conditional-form-actions-for-acfe' );
		$fields[] = $conditional;

		// --<
		return $fields;

	}

	/**
	 * Builds Redirect data array from mapped Fields.
	 *
	 * @since 0.1
	 *
	 * @param array   $form The array of Form data.
	 * @param integer $current_post_id The ID of the Post from which the Form has been submitted.
	 * @param string  $action The customised name of the action.
	 * @return array $data The array of Redirect data.
	 */
	public function form_redirect_data( $form, $current_post_id, $action ) {

		// Get some Form details.
		$form_name = acf_maybe_get( $form, 'name' );
		$form_id   = acf_maybe_get( $form, 'ID' );

		// Init data array.
		$data = [];

		// Get the Action variables.
		$url = get_sub_field( $this->field_key . 'redirect_url' );
		$url = acfe_form_map_field_value( $url, $current_post_id, $form );

		/**
		 * Filter the Redirect URL.
		 *
		 * @since 0.1
		 *
		 * @param string  $url The Redirect URL.
		 * @param array   $form The array of Form data.
		 * @param integer $current_post_id The ID of the Post from which the Form has been submitted.
		 * @param string  $action The customised name of the Form Action.
		 */
		$filter = 'acfe/form/submit/' . $this->action_name . '/url';
		$url    = apply_filters( $filter, $url, $form, $current_post_id, $action );
		$url    = apply_filters( $filter . '/form=' . $form_name, $url, $form, $current_post_id, $action );
		if ( ! empty( $action ) ) {
			$url = apply_filters( $filter . '/action=' . $action, $url, $form, $current_post_id, $action );
		}

		// Add trimmed URL to the data array.
		$data['redirect_url'] = trim( $url );

		// Get Redirect Conditional Reference.
		$data['redirect_conditional_ref'] = get_sub_field( $this->field_key . 'map_redirect_conditional' );
		$conditionals                     = [ $data['redirect_conditional_ref'] ];

		// Populate array with mapped Conditional Field values.
		$conditionals = acfe_form_map_vs_fields( $conditionals, $conditionals, $current_post_id, $form );

		// Save Redirect Conditional.
		$data['redirect_conditional'] = array_pop( $conditionals );

		// --<
		return $data;

	}

	/**
	 * Checks the Conditional Field given data from mapped Fields.
	 *
	 * @since 0.1.1
	 *
	 * @param array $redirect_data The array of Redirect data.
	 * @return bool $continue True if the Action should continue or false to skip.
	 */
	public function form_conditional_check( $redirect_data ) {

		// Approve by default.
		$continue = true;

		// Skip if the Redirect Conditional Reference Field has a value.
		if ( ! empty( $redirect_data['redirect_conditional_ref'] ) ) {
			// And the Redirect Conditional Field has no value.
			if ( empty( $redirect_data['redirect_conditional'] ) ) {
				return false;
			}
		}

		// --<
		return $continue;

	}

}
